@@ -81,9 +81,44 @@ following new ones.
 =head2 dequeue
 
   my $job_info = $backend->dequeue($worker_id, 0.5);
+  my $job_info = $backend->dequeue($worker_id, 0.5, {queues => ['important']});
 
 Wait for job, dequeue it and transition from C<inactive> to C<active> state or
-return C<undef> if queue was empty. Meant to be overloaded in a subclass.
+return C<undef> if queues were empty. Meant to be overloaded in a subclass.
+
+These options are currently available:
+
+=over 2
+
+=item queues
+
+  queues => ['important']
+
+One or more queues to dequeue jobs from, defaults to C<default>.
+
+=back
+
+These fields are currently available:
+
+=over 2
+
+=item args
+
+Job arguments.
+
+=item id
+
+Job ID.
+
+=item retries
+
+Number of times job has been retried.
+
+=item task
+
+Task name.
+
+=back
 
 =head2 enqueue
 
@@ -109,22 +144,29 @@ Delay job for this many seconds (from now).
 
 Job priority, defaults to C<0>.
 
+=item queue
+
+  queue => 'important'
+
+Queue to put job in, defaults to C<default>.
+
 =back
 
 =head2 fail_job
 
-  my $bool = $backend->fail_job($job_id);
-  my $bool = $backend->fail_job($job_id, 'Something went wrong!');
-  my $bool = $backend->fail_job($job_id, {msg => 'Something went wrong!'});
+  my $bool = $backend->fail_job($job_id, $retries);
+  my $bool = $backend->fail_job($job_id, $retries, 'Something went wrong!');
+  my $bool = $backend->fail_job(
+    $job_id, $retries, {msg => 'Something went wrong!'});
 
 Transition from C<active> to C<failed> state. Meant to be overloaded in a
 subclass.
 
 =head2 finish_job
 
-  my $bool = $backend->finish_job($job_id);
-  my $bool = $backend->finish_job($job_id, 'All went well!');
-  my $bool = $backend->finish_job($job_id, {msg => 'All went well!'});
+  my $bool = $backend->finish_job($job_id, $retries);
+  my $bool = $backend->finish_job($job_id, $retries, 'All went well!');
+  my $bool = $backend->finish_job($job_id, $retries, {msg => 'All went well!'});
 
 Transition from C<active> to C<finished> state. Meant to be overloaded in a
 subclass.
@@ -166,6 +208,10 @@ Time job was finished.
 
 Job priority.
 
+=item queue
+
+Queue name.
+
 =item result
 
 Job result.
@@ -184,7 +230,7 @@ Time job was started.
 
 =item state
 
-Current job state.
+Current job state, usually C<active>, C<failed>, C<finished> or C<inactive>.
 
 =item task
 
@@ -259,8 +305,8 @@ Reset job queue. Meant to be overloaded in a subclass.
 
 =head2 retry_job
 
-  my $bool = $backend->retry_job($job_id);
-  my $bool = $backend->retry_job($job_id, {delay => 10});
+  my $bool = $backend->retry_job($job_id, $retries);
+  my $bool = $backend->retry_job($job_id, $retries, {delay => 10});
 
 Transition from C<failed> or C<finished> state back to C<inactive>. Meant to be
 overloaded in a subclass.
@@ -275,6 +321,18 @@ These options are currently available:
 
 Delay job for this many seconds (from now).
 
+=item priority
+
+  priority => 5
+
+Job priority.
+
+=item queue
+
+  queue => 'important'
+
+Queue to put job in.
+
 =back
 
 =head2 stats
