@@ -14,9 +14,9 @@ sub db {
 }
 
 sub dequeue {
-  my ($self, $id, $timeout) = @_;
-  usleep($timeout * 1000000) unless my $job = $self->_try($id);
-  return $job || $self->_try($id);
+  my ($self, $id, $wait, $options) = @_;
+  usleep($wait * 1000000) unless my $job = $self->_try($id, $options);
+  return $job || $self->_try($id, $options);
 }
 
 sub enqueue {
@@ -31,6 +31,7 @@ sub enqueue {
     delayed => $options->{delay} ? (time + $options->{delay}) : 1,
     id      => $self->_id,
     priority => $options->{priority} // 0,
+    queue    => $options->{queue}    // 'default',
     retries  => 0,
     state    => 'inactive',
     task     => $task
@@ -120,14 +121,15 @@ sub repair {
 sub reset { $_[0]->db and delete($_[0]->{db})->clear }
 
 sub retry_job {
-  my ($self, $id) = (shift, shift);
+  my ($self, $id, $retries) = (shift, shift, shift);
   my $options = shift // {};
 
   my $guard = $self->_exclusive;
   return undef unless my $job = $self->_job($id, 'failed', 'finished');
+  return undef unless $job->{retries} == $retries;
   $job->{retries} += 1;
-  $job->{delayed}  = time + $options->{delay} if $options->{delay};
-  $job->{priority} = $options->{priority}     if defined $options->{priority};
+  $job->{delayed} = time + $options->{delay} if $options->{delay};
+  defined $options->{$_} and $job->{$_} = $options->{$_} for qw(priority queue);
   @$job{qw(retried state)} = (time, 'inactive');
   delete @$job{qw(finished result started worker)};
 
@@ -181,25 +183,28 @@ sub _job {
 sub _jobs { shift->db->{jobs} //= {} }
 
 sub _try {
-  my ($self, $id) = @_;
+  my ($self, $id, $options) = @_;
 
-  my $guard = $self->_exclusive;
-  my @ready = grep { $_->{state} eq 'inactive' } values %{$self->_jobs};
-  my $now   = time;
-  @ready = grep { $_->{delayed} < $now } @ready;
+  my $guard  = $self->_exclusive;
+  my @ready  = grep { $_->{state} eq 'inactive' } values %{$self->_jobs};
+  my %queues = map { $_ => 1 } @{$options->{queues} || ['default']};
+  my $tasks  = $self->minion->tasks;
+  @ready = grep { $queues{$_->{queue}} && $tasks->{$_->{task}} } @ready;
   @ready = sort { $a->{created} <=> $b->{created} } @ready;
   @ready = sort { $b->{priority} <=> $a->{priority} } @ready;
-  my $job = first { $self->minion->tasks->{$_->{task}} } @ready;
-  @$job{qw(started state worker)} = (time, 'active', $id) if $job;
 
-  return $job ? $job->export : undef;
+  my $now = time;
+  return undef unless my $job = first { $_->{delayed} < $now } @ready;
+  @$job{qw(started state worker)} = (time, 'active', $id);
+  return $job->export;
 }
 
 sub _update {
-  my ($self, $fail, $id, $result) = @_;
+  my ($self, $fail, $id, $retries, $result) = @_;
 
   my $guard = $self->_exclusive;
   return undef unless my $job = $self->_job($id, 'active');
+  return undef unless $job->{retries} == $retries;
   @$job{qw(finished result)} = (time, $result);
   $job->{state} = $fail ? 'failed' : 'finished';
   return 1;
@@ -259,9 +264,44 @@ L<DBM::Deep> object used to store all data.
 =head2 dequeue
 
   my $job_info = $backend->dequeue($worker_id, 0.5);
+  my $job_info = $backend->dequeue($worker_id, 0.5, {queues => ['important']});
 
 Wait for job, dequeue it and transition from C<inactive> to C<active> state or
-return C<undef> if queue was empty.
+return C<undef> if queues were empty.
+
+These options are currently available:
+
+=over 2
+
+=item queues
+
+  queues => ['important']
+
+One or more queues to dequeue jobs from, defaults to C<default>.
+
+=back
+
+These fields are currently available:
+
+=over 2
+
+=item args
+
+Job arguments.
+
+=item id
+
+Job ID.
+
+=item retries
+
+Number of times job has been retried.
+
+=item task
+
+Task name.
+
+=back
 
 =head2 enqueue
 
@@ -287,21 +327,28 @@ Delay job for this many seconds (from now).
 
 Job priority, defaults to C<0>.
 
+=item queue
+
+  queue => 'important'
+
+Queue to put job in, defaults to C<default>.
+
 =back
 
 =head2 fail_job
 
-  my $bool = $backend->fail_job($job_id);
-  my $bool = $backend->fail_job($job_id, 'Something went wrong!');
-  my $bool = $backend->fail_job($job_id, {msg => 'Something went wrong!'});
+  my $bool = $backend->fail_job($job_id, $retries);
+  my $bool = $backend->fail_job($job_id, $retries, 'Something went wrong!');
+  my $bool = $backend->fail_job(
+    $job_id, $retries, {msg => 'Something went wrong!'});
 
 Transition from C<active> to C<failed> state.
 
 =head2 finish_job
 
-  my $bool = $backend->finish_job($job_id);
-  my $bool = $backend->finish_job($job_id, 'All went well!');
-  my $bool = $backend->finish_job($job_id, {msg => 'All went well!'});
+  my $bool = $backend->finish_job($job_id, $retries);
+  my $bool = $backend->finish_job($job_id, $retries, 'All went well!');
+  my $bool = $backend->finish_job($job_id, $retries, {msg => 'All went well!'});
 
 Transition from C<active> to C<finished> state.
 
@@ -341,6 +388,10 @@ Time job was finished.
 
 Job priority.
 
+=item queue
+
+Queue name.
+
 =item result
 
 Job result.
@@ -359,7 +410,7 @@ Time job was started.
 
 =item state
 
-Current job state.
+Current job state, usually C<active>, C<failed>, C<finished> or C<inactive>.
 
 =item task
 
@@ -435,8 +486,8 @@ Reset job queue.
 
 =head2 retry_job
 
-  my $bool = $backend->retry_job($job_id);
-  my $bool = $backend->retry_job($job_id, {delay => 10});
+  my $bool = $backend->retry_job($job_id, $retries);
+  my $bool = $backend->retry_job($job_id, $retries, {delay => 10});
 
 Transition from C<failed> or C<finished> state back to C<inactive>.
 
@@ -456,6 +507,12 @@ Delay job for this many seconds (from now).
 
 Job priority.
 
+=item queue
+
+  queue => 'important'
+
+Queue to put job in.
+
 =back
 
 =head2 stats
