@@ -8,18 +8,19 @@ use Sys::Hostname 'hostname';
 has 'pg';
 
 sub dequeue {
-  my ($self, $id, $timeout) = @_;
+  my ($self, $id, $wait, $options) = @_;
 
-  if ((my $job = $self->_try($id)) || Mojo::IOLoop->is_running) { return $job }
+  if ((my $job = $self->_try($id, $options))) { return $job }
+  return undef if Mojo::IOLoop->is_running;
 
   my $db = $self->pg->db;
   $db->listen('minion.job')->on(notification => sub { Mojo::IOLoop->stop });
-  my $timer = Mojo::IOLoop->timer($timeout => sub { Mojo::IOLoop->stop });
+  my $timer = Mojo::IOLoop->timer($wait => sub { Mojo::IOLoop->stop });
   Mojo::IOLoop->start;
   $db->unlisten('*') and Mojo::IOLoop->remove($timer);
   undef $db;
 
-  return $self->_try($id);
+  return $self->_try($id, $options);
 }
 
 sub enqueue {
@@ -29,10 +30,10 @@ sub enqueue {
 
   my $db = $self->pg->db;
   return $db->query(
-    "insert into minion_jobs (args, delayed, priority, task)
-     values (?, (now() + (interval '1 second' * ?)), ?, ?)
+    "insert into minion_jobs (args, delayed, priority, queue, task)
+     values (?, (now() + (interval '1 second' * ?)), ?, ?, ?)
      returning id", {json => $args}, $options->{delay} // 0,
-    $options->{priority} // 0, $task
+    $options->{priority} // 0, $options->{queue} // 'default', $task
   )->hash->{id};
 }
 
@@ -43,7 +44,7 @@ sub job_info {
   shift->pg->db->query(
     'select id, args, extract(epoch from created) as created,
        extract(epoch from delayed) as delayed,
-       extract(epoch from finished) as finished, priority, result,
+       extract(epoch from finished) as finished, priority, queue, result,
        extract(epoch from retried) as retried, retries,
        extract(epoch from started) as started, state, task, worker
      from minion_jobs where id = ?', shift
@@ -128,17 +129,18 @@ sub repair {
 sub reset { shift->pg->db->query('truncate minion_jobs, minion_workers') }
 
 sub retry_job {
-  my ($self, $id) = (shift, shift);
+  my ($self, $id, $retries) = (shift, shift, shift);
   my $options = shift // {};
 
   return !!$self->pg->db->query(
     "update minion_jobs
-     set finished = null, priority = coalesce(?, priority), result = null,
-       retried = now(), retries = retries + 1, started = null,
-       state = 'inactive', worker = null,
+     set finished = null, priority = coalesce(?, priority),
+       queue = coalesce(?, queue), result = null, retried = now(),
+       retries = retries + 1, started = null, state = 'inactive', worker = null,
        delayed = (now() + (interval '1 second' * ?))
-     where id = ? and state in ('failed', 'finished')
-     returning 1", $options->{priority}, $options->{delay} // 0, $id
+     where id = ? and retries = ? and state in ('failed', 'finished')
+     returning 1", @$options{qw(priority queue)}, $options->{delay} // 0, $id,
+    $retries
   )->rows;
 }
 
@@ -181,30 +183,33 @@ sub worker_info {
 }
 
 sub _try {
-  my ($self, $id) = @_;
+  my ($self, $id, $options) = @_;
 
   return $self->pg->db->query(
     "update minion_jobs
      set started = now(), state = 'active', worker = ?
      where id = (
        select id from minion_jobs
-       where state = 'inactive' and delayed < now() and task = any (?)
+       where delayed < now() and queue = any (?) and state = 'inactive'
+         and task = any (?)
        order by priority desc, created
        limit 1
        for update
      )
-     returning id, args, task", $id, [keys %{$self->minion->tasks}]
+     returning id, args, retries, task", $id,
+    $options->{queues} || ['default'], [keys %{$self->minion->tasks}]
   )->expand->hash;
 }
 
 sub _update {
-  my ($self, $fail, $id, $result) = @_;
+  my ($self, $fail, $id, $retries, $result) = @_;
 
   return !!$self->pg->db->query(
     "update minion_jobs
      set finished = now(), result = ?, state = ?
-     where id = ? and state = 'active'
-     returning 1", {json => $result}, $fail ? 'failed' : 'finished', $id
+     where id = ? and retries = ? and state = 'active'
+     returning 1", {json => $result}, $fail ? 'failed' : 'finished', $id,
+    $retries
   )->rows;
 }
 
@@ -249,9 +254,44 @@ implements the following new ones.
 =head2 dequeue
 
   my $job_info = $backend->dequeue($worker_id, 0.5);
+  my $job_info = $backend->dequeue($worker_id, 0.5, {queues => ['important']});
 
 Wait for job, dequeue it and transition from C<inactive> to C<active> state or
-return C<undef> if queue was empty.
+return C<undef> if queues were empty.
+
+These options are currently available:
+
+=over 2
+
+=item queues
+
+  queues => ['important']
+
+One or more queues to dequeue jobs from, defaults to C<default>.
+
+=back
+
+These fields are currently available:
+
+=over 2
+
+=item args
+
+Job arguments.
+
+=item id
+
+Job ID.
+
+=item retries
+
+Number of times job has been retried.
+
+=item task
+
+Task name.
+
+=back
 
 =head2 enqueue
 
@@ -277,21 +317,28 @@ Delay job for this many seconds (from now).
 
 Job priority, defaults to C<0>.
 
+=item queue
+
+  queue => 'important'
+
+Queue to put job in, defaults to C<default>.
+
 =back
 
 =head2 fail_job
 
-  my $bool = $backend->fail_job($job_id);
-  my $bool = $backend->fail_job($job_id, 'Something went wrong!');
-  my $bool = $backend->fail_job($job_id, {msg => 'Something went wrong!'});
+  my $bool = $backend->fail_job($job_id, $retries);
+  my $bool = $backend->fail_job($job_id, $retries, 'Something went wrong!');
+  my $bool = $backend->fail_job(
+    $job_id, $retries, {msg => 'Something went wrong!'});
 
 Transition from C<active> to C<failed> state.
 
 =head2 finish_job
 
-  my $bool = $backend->finish_job($job_id);
-  my $bool = $backend->finish_job($job_id, 'All went well!');
-  my $bool = $backend->finish_job($job_id, {msg => 'All went well!'});
+  my $bool = $backend->finish_job($job_id, $retries);
+  my $bool = $backend->finish_job($job_id, $retries, 'All went well!');
+  my $bool = $backend->finish_job($job_id, $retries, {msg => 'All went well!'});
 
 Transition from C<active> to C<finished> state.
 
@@ -331,6 +378,10 @@ Time job was finished.
 
 Job priority.
 
+=item queue
+
+Queue name.
+
 =item result
 
 Job result.
@@ -349,7 +400,7 @@ Time job was started.
 
 =item state
 
-Current job state.
+Current job state, usually C<active>, C<failed>, C<finished> or C<inactive>.
 
 =item task
 
@@ -425,8 +476,8 @@ Reset job queue.
 
 =head2 retry_job
 
-  my $bool = $backend->retry_job($job_id);
-  my $bool = $backend->retry_job($job_id, {delay => 10});
+  my $bool = $backend->retry_job($job_id, $retries);
+  my $bool = $backend->retry_job($job_id, $retries, {delay => 10});
 
 Transition from C<failed> or C<finished> state back to C<inactive>.
 
@@ -446,6 +497,12 @@ Delay job for this many seconds (from now).
 
 Job priority.
 
+=item queue
+
+  queue => 'important'
+
+Queue to put job in.
+
 =back
 
 =head2 stats
@@ -554,3 +611,6 @@ alter table minion_workers alter column started set default now();
 
 -- 3 up
 create index on minion_jobs (state);
+
+-- 4 up
+alter table minion_jobs add column queue text not null default 'default';
