@@ -4,21 +4,21 @@ use Mojo::Base 'Mojo::EventEmitter';
 use Mojo::IOLoop;
 use POSIX 'WNOHANG';
 
-has args => sub { [] };
-has [qw(id minion task)];
+has [qw(args id minion retries task)];
 
 sub app { shift->minion->app }
 
 sub fail {
   my $self = shift;
   my $err  = shift // 'Unknown error';
-  my $ok   = $self->minion->backend->fail_job($self->id, $err);
+  my $ok   = $self->minion->backend->fail_job($self->id, $self->retries, $err);
   return $ok ? !!$self->emit(failed => $err) : undef;
 }
 
 sub finish {
   my ($self, $result) = @_;
-  my $ok = $self->minion->backend->finish_job($self->id, $result);
+  my $ok
+    = $self->minion->backend->finish_job($self->id, $self->retries, $result);
   return $ok ? !!$self->emit(finished => $result) : undef;
 }
 
@@ -39,7 +39,10 @@ sub perform {
 
 sub remove { $_[0]->minion->backend->remove_job($_[0]->id) }
 
-sub retry { $_[0]->minion->backend->retry_job($_[0]->id, $_[1]) }
+sub retry {
+  my $self = shift;
+  return $self->minion->backend->retry_job($self->id, $self->retries, @_);
+}
 
 sub start {
   my $self = shift;
@@ -155,6 +158,13 @@ Job id.
 
 L<Minion> object this job belongs to.
 
+=head2 retries
+
+  my $retries = $job->retries;
+  $job        = $job->retries(5);
+
+Number of times job has been retried.
+
 =head2 task
 
   my $task = $job->task;
@@ -228,6 +238,10 @@ Time job was finished.
 
 Job priority.
 
+=item queue
+
+Queue name.
+
 =item result
 
 Job result.
@@ -246,7 +260,7 @@ Time job was started.
 
 =item state
 
-Current job state.
+Current job state, usually C<active>, C<failed>, C<finished> or C<inactive>.
 
 =item task
 
@@ -299,6 +313,12 @@ Delay job for this many seconds (from now).
 
 Job priority.
 
+=item queue
+
+  queue => 'important'
+
+Queue to put job in.
+
 =back
 
 =head2 start
