@@ -4,18 +4,19 @@ use Mojo::Base 'Mojo::EventEmitter';
 has [qw(id minion)];
 
 sub dequeue {
-  my ($self, $timeout) = @_;
+  my ($self, $wait, $options) = @_;
 
   # Worker not registered
   return undef unless my $id = $self->id;
 
   my $minion = $self->minion;
-  return undef unless my $job = $minion->backend->dequeue($id, $timeout);
+  return undef unless my $job = $minion->backend->dequeue($id, $wait, $options);
   $job = Minion::Job->new(
-    args   => $job->{args},
-    id     => $job->{id},
-    minion => $minion,
-    task   => $job->{task}
+    args    => $job->{args},
+    id      => $job->{id},
+    minion  => $minion,
+    retries => $job->{retries},
+    task    => $job->{task}
   );
   $self->emit(dequeue => $job);
   return $job;
@@ -95,9 +96,22 @@ implements the following new ones.
 =head2 dequeue
 
   my $job = $worker->dequeue(0.5);
+  my $job = $worker->dequeue(0.5 => {queues => ['important']});
 
 Wait for job, dequeue L<Minion::Job> object and transition from C<inactive> to
-C<active> state or return C<undef> if queue was empty.
+C<active> state or return C<undef> if queues were empty.
+
+These options are currently available:
+
+=over 2
+
+=item queues
+
+  queues => ['important']
+
+One or more queues to dequeue jobs from, defaults to C<default>.
+
+=back
 
 =head2 info
 
